(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    382711,       7179]
NotebookOptionsPosition[    381189,       7127]
NotebookOutlinePosition[    381874,       7151]
CellTagsIndexPosition[    381831,       7148]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Plots for the Running of the Mixing Parameters and Masses", "Title"],

Cell[TextData[{
 "This notebook can be used to create plots of the RG evolution, ",
 StyleBox["after",
  FontWeight->"Bold"],
 StyleBox[" the RGEs have been solved (RGESolve) in a different notebook.",
  FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{"Some", " ", "preliminary", " ", "graphics", " ", "definitions"}],
     " ", "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Needs", "[", "\"\<REAP`RGEPlotUtilities`\>\"", "]"}], 
   ";"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$VersionNumber", "<", "6"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Get", "[", "\"\<Graphics`FilledPlot`\>\"", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Get", "[", "\"\<Graphics`Colors`\>\"", "]"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ForestGreen", "=", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.1", ",", "0.75", ",", "0.2"}], "]"}]}], ";"}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.435587848557613*^9, 3.435587856517888*^9}, {
  3.435655531948548*^9, 3.435655544087432*^9}, {3.635817589174589*^9, 
  3.63581759434683*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{"Define", " ", "the", " ", "plot", " ", "region"}], " ", "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"ClearAll", "[", 
    RowBox[{"tmin", ",", "tmax"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"tmin", "=", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "100"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"tmax", "=", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", 
       RowBox[{"2", "*", 
        RowBox[{"10", "^", "16"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   StyleBox[
    RowBox[{"(*", " ", 
     RowBox[{
     "Define", " ", "the", " ", "functions", " ", "that", " ", "should", " ", 
      "be", " ", "plotted"}], " ", "*)"}],
    FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
   RowBox[{"ClearAll", "[", 
    RowBox[{"Mnu", ",", "Ye"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Mnu", "[", "mu_", "]"}], ":=", 
    RowBox[{
     RowBox[{"RGEGetSolution", "[", 
      RowBox[{"mu", ",", "RGEM\[Nu]"}], "]"}], "//", "Chop"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Ye", "[", "mu_", "]"}], ":=", 
    RowBox[{
     RowBox[{"RGEGetSolution", "[", 
      RowBox[{"mu", ",", "RGEYe"}], "]"}], "//", "Chop"}]}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"ClearAll", "[", 
    RowBox[{
    "MixingPars", ",", "MixingParsLogScale", ",", "NuMasses", ",", 
     "NuMassesLogScale"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"MixingPars", "[", "mu_", "]"}], ":=", 
    RowBox[{
     RowBox[{"MNSParameters", "[", 
      RowBox[{
       RowBox[{"Mnu", "[", "mu", "]"}], ",", 
       RowBox[{"Ye", "[", "mu", "]"}]}], "]"}], "\[LeftDoubleBracket]", "1", 
     "\[RightDoubleBracket]"}]}], "\n", 
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "x_", "]"}], ":=", 
    RowBox[{"MixingPars", "[", 
     RowBox[{"10", "^", "x"}], "]"}]}], "\n", 
   RowBox[{
    RowBox[{"NuMasses", "[", "mu_", "]"}], ":=", 
    RowBox[{
     RowBox[{"MNSParameters", "[", 
      RowBox[{
       RowBox[{"Mnu", "[", "mu", "]"}], ",", 
       RowBox[{"Ye", "[", "mu", "]"}]}], "]"}], "\[LeftDoubleBracket]", "2", 
     "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"NuMassesLogScale", "[", "x_", "]"}], ":=", 
    RowBox[{"NuMasses", "[", 
     RowBox[{"10", "^", "x"}], "]"}]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"ClearAll", "[", 
    RowBox[{
    "\[Theta]12", ",", "\[Theta]13", ",", "\[Theta]23", ",", "\[Delta]", ",", 
     "\[CurlyPhi]1", ",", "\[CurlyPhi]2", ",", "m1", ",", "m2", ",", "m3", 
     ",", "\[CapitalDelta]m2sol", ",", "\[CapitalDelta]m2atm"}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Theta]12", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
      "1", "\[RightDoubleBracket]"}], "/", "Degree"}]}], "\n", 
   RowBox[{
    RowBox[{"\[Theta]13", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
      "2", "\[RightDoubleBracket]"}], "/", "Degree"}]}], "\n", 
   RowBox[{
    RowBox[{"\[Theta]23", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
      "3", "\[RightDoubleBracket]"}], "/", "Degree"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Delta]", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
      "4", "\[RightDoubleBracket]"}], "/", "Degree"}]}], "\n", 
   RowBox[{
    RowBox[{"\[CurlyPhi]1", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
      "8", "\[RightDoubleBracket]"}], "/", "Degree"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[CurlyPhi]2", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
      "9", "\[RightDoubleBracket]"}], "/", "Degree"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"m1", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{"NuMassesLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", "1",
      "\[RightDoubleBracket]"}]}], "\n", 
   RowBox[{
    RowBox[{"m2", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{"NuMassesLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", "2",
      "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"m3", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{"NuMassesLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", "3",
      "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[CapitalDelta]m2sol", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"m2", "[", "t", "]"}], "^", "2"}], "-", 
     RowBox[{
      RowBox[{"m1", "[", "t", "]"}], "^", "2"}]}]}], "\n", 
   RowBox[{
    RowBox[{"\[CapitalDelta]m2atm", "[", "t_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"m3", "[", "t", "]"}], "^", "2"}], "-", 
     RowBox[{
      RowBox[{"m2", "[", "t", "]"}], "^", "2"}]}]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   StyleBox[
    RowBox[{"(*", " ", 
     RowBox[{"Positions", " ", "of", " ", "the", " ", "thresholds"}], " ", 
     "*)"}],
    FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
   RowBox[{"ClearAll", "[", 
    RowBox[{"M1", ",", "M2", ",", "M3", ",", "ShadowEFT"}], "]"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"M1", ",", "M2", ",", "M3"}], "}"}], "=", 
     RowBox[{
      RowBox[{"RGEGetTransitions", "[", "]"}], "\[LeftDoubleBracket]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"4", ",", "3", ",", "2"}], "}"}], ",", "1"}], 
      "\[RightDoubleBracket]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ShadowEFT", "=", 
     RowBox[{"RGEShadowEFT", "[", 
      RowBox[{"M1", ",", "M2", ",", "M3", ",", 
       SuperscriptBox["10", "17"]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{
    "Draw", " ", "the", " ", "plot", " ", "for", " ", "the", " ", "mixing", 
     " ", "angles"}], " ", "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Plot", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Theta]12", "[", "t", "]"}], ",", 
        RowBox[{"\[Theta]13", "[", "t", "]"}], ",", 
        RowBox[{"\[Theta]23", "[", "t", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
      RowBox[{"FrameLabel", "->", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
         "\"\<\>\""}], "}"}]}], ",", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"RGELogTicksLabeled", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "*", "15"}], ",", 
             RowBox[{
              RowBox[{"ToString", "[", 
               RowBox[{"i", "*", "15"}], "]"}], "<>", "\"\<\[Degree]\>\""}]}],
             "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "3"}], "}"}]}], "]"}], ",", 
         RowBox[{"RGELogTicks", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "*", "15"}], ",", "\"\<\>\""}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "3"}], "}"}]}], "]"}]}], "}"}]}], ",",
       "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"All", ",", "All"}], "}"}]}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Red", ",", "ForestGreen", ",", "Blue"}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Prolog", "\[Rule]", 
       RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"Epilog", "\[Rule]", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubscriptBox["\[Theta]", "12"], ",", 
             RowBox[{"FontColor", "\[Rule]", "Red"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"\[Theta]12", "[", "tmin", "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubscriptBox["\[Theta]", "13"], ",", 
             RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"\[Theta]13", "[", "tmin", "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", 
             RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubscriptBox["\[Theta]", "23"], ",", 
             RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"\[Theta]23", "[", "tmin", "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}]}], "}"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
     "\[IndentingNewLine]", "]"}], ",", 
    RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.436085031323138*^9, 3.436085044178268*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[1, 0, 0], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwd0nk0VesbB3C5OOfITJFMZQqbTsbY+5zzTUKG3GQebjeRIW79qFSGkORy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     "]]}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0nk4lN0bB3AJY0uhkiVkX8bMgyKi+27yIj9aRZKKaJMU0SKFxlJCCkXx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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwd1Hk01tkfB3AMnseabYhsWR7bF08Mpci70vex5lcj0io1E6WSFkUkPVlD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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 0},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[Theta]", "12"], FontColor -> RGBColor[1, 0, 0], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[1, 0, 0], FontSize -> 12]& ], 
      TraditionalForm], {2, 32.49477869949709}, {-0.8, 1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[Theta]", "13"], FontColor -> RGBColor[0.1, 0.75, 0.2],
         FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {2, 0.10085322634288943`}, {-0.8, -1}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[Theta]", "23"], FontColor -> RGBColor[0, 0, 1], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {2, 46.96199368321047}, {-0.8, 1.4}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox["\"\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{0, 
       FormBox["\"0\[Degree]\"", TraditionalForm]}, {15, 
       FormBox["\"15\[Degree]\"", TraditionalForm]}, {30, 
       FormBox["\"30\[Degree]\"", TraditionalForm]}, {45, 
       FormBox["\"45\[Degree]\"", TraditionalForm]}}, {{0, 
       FormBox["\"\"", TraditionalForm]}, {15, 
       FormBox["\"\"", TraditionalForm]}, {30, 
       FormBox["\"\"", TraditionalForm]}, {45, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847366954854*^9, 3.63684744315666*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{
    "Draw", " ", "the", " ", "plot", " ", "for", " ", "the", " ", "phases"}], 
    " ", "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Plot", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", "[", "t", "]"}], ",", 
        RowBox[{"\[CurlyPhi]1", "[", "t", "]"}], ",", 
        RowBox[{"\[CurlyPhi]2", "[", "t", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
      RowBox[{"FrameLabel", "->", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
         "\"\<\>\""}], "}"}]}], ",", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"RGELogTicksLabeled", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "*", "45"}], ",", 
             RowBox[{
              RowBox[{"ToString", "[", 
               RowBox[{"i", "*", "45"}], "]"}], "<>", "\"\<\[Degree]\>\""}]}],
             "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "8"}], "}"}]}], "]"}], ",", 
         RowBox[{"RGELogTicks", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "*", "45"}], ",", "\"\<\>\""}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "12"}], "}"}]}], "]"}]}], "}"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"All", ",", "All"}], "}"}]}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Red", ",", "Blue", ",", "ForestGreen"}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Prolog", "\[Rule]", 
       RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"Epilog", "\[Rule]", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{"\[Delta]", ",", 
             RowBox[{"FontColor", "\[Rule]", "Red"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"\[Delta]", "[", "tmin", "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubscriptBox["\[CurlyPhi]", "1"], ",", 
             RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"\[CurlyPhi]1", "[", "tmin", "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubscriptBox["\[CurlyPhi]", "2"], ",", 
             RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"\[CurlyPhi]2", "[", "tmin", "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", 
             RowBox[{"-", "1"}]}], "}"}]}], "]"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
     "\[IndentingNewLine]", "]"}], ",", 
    RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.436085053698209*^9, 3.436085057270277*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[1, 0, 0], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0nlYzAsXB/BEzZSiVYtUtGn6Teb9pWXafE2Z35QWUtMq3YlEkYrboijm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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwl1Hk81XkXB3Bdxr1X5bGNfcty5d7f1W9oodDJ3Eey1JMRKZRbDWmhnqlE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     "]]}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwd1Hs41NkfB/BZiZlKuW0uuUUuMTQ/SpR8TmWZRG2lUqaLSUIRupAiaRoi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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 0},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox["\[Delta]", FontColor -> RGBColor[1, 0, 0], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[1, 0, 0], FontSize -> 12]& ], 
      TraditionalForm], {2, 18.357855024213087`}, {-0.8, 1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[CurlyPhi]", "1"], FontColor -> RGBColor[0, 0, 1], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {2, 359.93380537334815`}, {-0.8, 1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[CurlyPhi]", "2"], FontColor -> 
        RGBColor[0.1, 0.75, 0.2], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {2, 0.02287543073700604}, {-0.8, -1}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox["\"\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{0, 
       FormBox["\"0\[Degree]\"", TraditionalForm]}, {45, 
       FormBox["\"45\[Degree]\"", TraditionalForm]}, {90, 
       FormBox["\"90\[Degree]\"", TraditionalForm]}, {135, 
       FormBox["\"135\[Degree]\"", TraditionalForm]}, {180, 
       FormBox["\"180\[Degree]\"", TraditionalForm]}, {225, 
       FormBox["\"225\[Degree]\"", TraditionalForm]}, {270, 
       FormBox["\"270\[Degree]\"", TraditionalForm]}, {315, 
       FormBox["\"315\[Degree]\"", TraditionalForm]}, {360, 
       FormBox["\"360\[Degree]\"", TraditionalForm]}}, {{0, 
       FormBox["\"\"", TraditionalForm]}, {45, 
       FormBox["\"\"", TraditionalForm]}, {90, 
       FormBox["\"\"", TraditionalForm]}, {135, 
       FormBox["\"\"", TraditionalForm]}, {180, 
       FormBox["\"\"", TraditionalForm]}, {225, 
       FormBox["\"\"", TraditionalForm]}, {270, 
       FormBox["\"\"", TraditionalForm]}, {315, 
       FormBox["\"\"", TraditionalForm]}, {360, 
       FormBox["\"\"", TraditionalForm]}, {405, 
       FormBox["\"\"", TraditionalForm]}, {450, 
       FormBox["\"\"", TraditionalForm]}, {495, 
       FormBox["\"\"", TraditionalForm]}, {540, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847367087133*^9, 3.636847443831415*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{
    "Draw", " ", "the", " ", "plot", " ", "for", " ", "the", " ", "mass", " ",
      "eigenvalues"}], " ", "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"tlabel", "=", "4"}], ";"}], "   ", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Position", " ", "where", " ", "the", " ", "labels", " ", "m1"}],
      ",", "m2", ",", 
     RowBox[{"m3", " ", "should", " ", "be", " ", "placed"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"m1", "[", "t", "]"}], ",", 
         RowBox[{"m2", "[", "t", "]"}], ",", 
         RowBox[{"m3", "[", "t", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
       RowBox[{"FrameLabel", "->", 
        RowBox[{"{", 
         RowBox[{
         "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
          "\"\<\!\(\*SubscriptBox[\(m\), \(i\)]\) [eV]\>\""}], "}"}]}], ",", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"RGELogTicksLabeled", "[", 
           RowBox[{"2", ",", "19"}], "]"}], ",", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"i", "*", "0.01"}], ",", 
              RowBox[{"PaddedForm", "[", 
               RowBox[{
                RowBox[{"i", "*", "0.01"}], ",", 
                RowBox[{"{", 
                 RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", "50"}], "}"}]}], "]"}], ",", 
          RowBox[{"RGELogTicks", "[", 
           RowBox[{"2", ",", "19"}], "]"}], ",", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"i", "*", "0.01"}], ",", "\"\<\>\""}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", "50"}], "}"}]}], "]"}]}], "}"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"All", ",", "All"}], "}"}]}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Red", ",", "Blue", ",", "ForestGreen"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Prolog", "\[Rule]", 
        RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"Epilog", "\[Rule]", 
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Text", "[", 
           RowBox[{
            RowBox[{"StyleForm", "[", 
             RowBox[{
              SubscriptBox["m", "1"], ",", 
              RowBox[{"FontColor", "\[Rule]", "Red"}], ",", 
              RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"tlabel", ",", 
              RowBox[{"m1", "[", "tlabel", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Text", "[", 
           RowBox[{
            RowBox[{"StyleForm", "[", 
             RowBox[{
              SubscriptBox["m", "2"], ",", 
              RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
              RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"tlabel", ",", 
              RowBox[{"m2", "[", "tlabel", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "0.8"}], ",", 
              RowBox[{"-", "2"}]}], "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Text", "[", 
           RowBox[{
            RowBox[{"StyleForm", "[", 
             RowBox[{
              SubscriptBox["m", "3"], ",", 
              RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
              RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"tlabel", ",", 
              RowBox[{"m3", "[", "tlabel", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}]}], "}"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
      "\[IndentingNewLine]", "]"}], ",", 
     RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.436085066913046*^9, 3.436085067704411*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[1, 0, 0], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k4lOsbB/DhwgyhQdmSncnMi0khhefmaDgip5ItOaXFUh1lSSmFJiTG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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0ns81OkXB3ArZoawbrlHJGrmS5NblO2cJLfc1iq0tLlUUkm6KKVUQyJk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     "]]}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k0ldsbB/CTDgepDN0IIWMdL500mD1PkimafqIkdclNVKRCKiQdOsRN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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 0.012},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["m", "1"], FontColor -> RGBColor[1, 0, 0], FontSize -> 
        12], StyleForm[#, FontColor -> RGBColor[1, 0, 0], FontSize -> 12]& ], 
      TraditionalForm], {4, 0.01661933208461064}, {-0.8, 1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["m", "2"], FontColor -> RGBColor[0, 0, 1], FontSize -> 
        12], StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {4, 0.0193330121557749}, {-0.8, -2}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["m", "3"], FontColor -> RGBColor[0.1, 0.75, 0.2], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {4, 0.054821183820263436`}, {-0.8, 1.4}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(m\\), \\(i\\)]\\) [eV]\"", TraditionalForm], 
     None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{0., 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.00\"", 0., AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.01, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.01\"", 0.01, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.02, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.02\"", 0.02, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.03, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.03\"", 0.03, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.04, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.04\"", 0.04, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.05, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.05\"", 0.05, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.06, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.06\"", 0.06, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.07, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.07\"", 0.07, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.08, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.08\"", 0.08, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.09, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.09\"", 0.09, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.1, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.10\"", 0.1, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.11, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.11\"", 0.11, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.12, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.12\"", 0.12, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.13, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.13\"", 0.13, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.14, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.14\"", 0.14, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.15, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.15\"", 0.15, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.16, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.16\"", 0.16, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.17, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.17\"", 0.17, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.18, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.18\"", 0.18, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.19, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.19\"", 0.19, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.20\"", 0.2, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.21, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.21\"", 0.21, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.22, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.22\"", 0.22, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.23, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.23\"", 0.23, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.24, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.24\"", 0.24, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.25, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.25\"", 0.25, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.26, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.26\"", 0.26, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.27, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.27\"", 0.27, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.28, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.28\"", 0.28, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.29, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.29\"", 0.29, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.3, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.30\"", 0.3, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.31, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.31\"", 0.31, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.32, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.32\"", 0.32, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.33, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.33\"", 0.33, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.34, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.34\"", 0.34, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.35000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.35\"", 0.35000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.36, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.36\"", 0.36, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.37, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.37\"", 0.37, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.38, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.38\"", 0.38, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.39, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.39\"", 0.39, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.40\"", 0.4, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.41000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.41\"", 0.41000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.42, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.42\"", 0.42, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.43, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.43\"", 0.43, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.44, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.44\"", 0.44, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.45, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.45\"", 0.45, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.46, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.46\"", 0.46, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.47000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.47\"", 0.47000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.48, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.48\"", 0.48, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.49, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.49\"", 0.49, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.5, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.50\"", 0.5, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}}, {{0., 
       FormBox["\"\"", TraditionalForm]}, {0.01, 
       FormBox["\"\"", TraditionalForm]}, {0.02, 
       FormBox["\"\"", TraditionalForm]}, {0.03, 
       FormBox["\"\"", TraditionalForm]}, {0.04, 
       FormBox["\"\"", TraditionalForm]}, {0.05, 
       FormBox["\"\"", TraditionalForm]}, {0.06, 
       FormBox["\"\"", TraditionalForm]}, {0.07, 
       FormBox["\"\"", TraditionalForm]}, {0.08, 
       FormBox["\"\"", TraditionalForm]}, {0.09, 
       FormBox["\"\"", TraditionalForm]}, {0.1, 
       FormBox["\"\"", TraditionalForm]}, {0.11, 
       FormBox["\"\"", TraditionalForm]}, {0.12, 
       FormBox["\"\"", TraditionalForm]}, {0.13, 
       FormBox["\"\"", TraditionalForm]}, {0.14, 
       FormBox["\"\"", TraditionalForm]}, {0.15, 
       FormBox["\"\"", TraditionalForm]}, {0.16, 
       FormBox["\"\"", TraditionalForm]}, {0.17, 
       FormBox["\"\"", TraditionalForm]}, {0.18, 
       FormBox["\"\"", TraditionalForm]}, {0.19, 
       FormBox["\"\"", TraditionalForm]}, {0.2, 
       FormBox["\"\"", TraditionalForm]}, {0.21, 
       FormBox["\"\"", TraditionalForm]}, {0.22, 
       FormBox["\"\"", TraditionalForm]}, {0.23, 
       FormBox["\"\"", TraditionalForm]}, {0.24, 
       FormBox["\"\"", TraditionalForm]}, {0.25, 
       FormBox["\"\"", TraditionalForm]}, {0.26, 
       FormBox["\"\"", TraditionalForm]}, {0.27, 
       FormBox["\"\"", TraditionalForm]}, {0.28, 
       FormBox["\"\"", TraditionalForm]}, {0.29, 
       FormBox["\"\"", TraditionalForm]}, {0.3, 
       FormBox["\"\"", TraditionalForm]}, {0.31, 
       FormBox["\"\"", TraditionalForm]}, {0.32, 
       FormBox["\"\"", TraditionalForm]}, {0.33, 
       FormBox["\"\"", TraditionalForm]}, {0.34, 
       FormBox["\"\"", TraditionalForm]}, {0.35000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {0.36, 
       FormBox["\"\"", TraditionalForm]}, {0.37, 
       FormBox["\"\"", TraditionalForm]}, {0.38, 
       FormBox["\"\"", TraditionalForm]}, {0.39, 
       FormBox["\"\"", TraditionalForm]}, {0.4, 
       FormBox["\"\"", TraditionalForm]}, {0.41000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {0.42, 
       FormBox["\"\"", TraditionalForm]}, {0.43, 
       FormBox["\"\"", TraditionalForm]}, {0.44, 
       FormBox["\"\"", TraditionalForm]}, {0.45, 
       FormBox["\"\"", TraditionalForm]}, {0.46, 
       FormBox["\"\"", TraditionalForm]}, {0.47000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {0.48, 
       FormBox["\"\"", TraditionalForm]}, {0.49, 
       FormBox["\"\"", TraditionalForm]}, {0.5, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847367217153*^9, 3.636847444866641*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{
    "Draw", " ", "the", " ", "plot", " ", "for", " ", "the", " ", "mass", " ",
      "squared", " ", "differences", " ", 
     RowBox[{"(", 
      RowBox[{"on", " ", "a", " ", "logarithmic", " ", "scale"}], ")"}]}], 
    " ", "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Plot", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Log", "[", 
         RowBox[{"10", ",", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[CapitalDelta]m2sol", "[", "t", "]"}], "]"}]}], "]"}], 
        ",", 
        RowBox[{"Log", "[", 
         RowBox[{"10", ",", 
          RowBox[{"Abs", "[", 
           RowBox[{"\[CapitalDelta]m2atm", "[", "t", "]"}], "]"}]}], "]"}]}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
      RowBox[{"FrameLabel", "->", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
         "\"\<\!\(\*SubsuperscriptBox[\(\[CapitalDelta]m\), \(i\), \(2\)]\) [\
\!\(\*SuperscriptBox[\(eV\), \(2\)]\)]\>\""}], "}"}]}], ",", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"RGELogTicksLabeled", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"RGELogTicksLabeledNegExp", "[", 
          RowBox[{
           RowBox[{"-", "6"}], ",", 
           RowBox[{"-", "1"}]}], "]"}], ",", 
         RowBox[{"RGELogTicks", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"RGELogTicks", "[", 
          RowBox[{
           RowBox[{"-", "6"}], ",", 
           RowBox[{"-", "1"}]}], "]"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"All", ",", "All"}], "}"}]}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ForestGreen", ",", "Blue"}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Prolog", "\[Rule]", 
       RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"Epilog", "\[Rule]", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubsuperscriptBox["\[CapitalDelta]m", "sol", "2"], ",", 
             RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"Log", "[", 
              RowBox[{"10", ",", 
               RowBox[{"Abs", "[", 
                RowBox[{"\[CapitalDelta]m2sol", "[", "tmin", "]"}], "]"}]}], 
              "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", 
             RowBox[{"-", "2"}]}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Text", "[", 
          RowBox[{
           RowBox[{"StyleForm", "[", 
            RowBox[{
             SubsuperscriptBox["\[CapitalDelta]m", "atm", "2"], ",", 
             RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
             RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"tmin", ",", 
             RowBox[{"Log", "[", 
              RowBox[{"10", ",", 
               RowBox[{"Abs", "[", 
                RowBox[{"\[CapitalDelta]m2atm", "[", "tmin", "]"}], "]"}]}], 
              "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}]}], "}"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
     "\[IndentingNewLine]", "]"}], ",", 
    RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.436085078169314*^9, 3.436085079707715*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0ns41GkbwPFJzIyY3/xQOZ/CYPwwL8k599ppHEJXElE6TEe2g2yllEIT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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0ns8lPkXwPHJ84xxGbtI7kmEjWd4FhGJQ0xWbpXGJbGNFCpaChFJ85OU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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., -4.2},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubsuperscriptBox["\[CapitalDelta]m", "sol", "2"], FontColor -> 
        RGBColor[0.1, 0.75, 0.2], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {2, -4.109350063293438}, {-0.8, -2}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubsuperscriptBox["\[CapitalDelta]m", "atm", "2"], FontColor -> 
        RGBColor[0, 0, 1], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {2, -2.6762234834607406`}, {-0.8, 1.4}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubsuperscriptBox[\\(\[CapitalDelta]m\\), \\(i\\), \
\\(2\\)]\\) [\\!\\(\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)]\"", 
      TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{-6, 
       FormBox["\"\\!\\(10\\^\\(-6\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500000], -5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 3]], -5.5228787452803365`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250000], -5.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200000], -5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500000, 3]], -5.221848749616355], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 7]], -5.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125000], -5.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 9]], -5.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-5, 
       FormBox["\"\\!\\(10\\^\\(-5\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50000], -4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 3]], -4.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25000], -4.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20000], -4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50000, 3]], -4.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 7]], -4.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[12500], -4.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 9]], -4.045757490560674], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-4, 
       FormBox["\"\\!\\(10\\^\\(-4\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[5000], -3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 3]], -3.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2500], -3.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2000], -3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[5000, 3]], -3.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 7]], -3.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[1250], -3.096910013008056], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 9]], -3.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-3, 
       FormBox["\"\\!\\(10\\^\\(-3\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500], -2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 3]], -2.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250], -2.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200], -2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500, 3]], -2.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 7]], -2.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125], -2.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 9]], -2.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-2, 
       FormBox["\"\\!\\(10\\^\\(-2\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50], -1.6989700043360185`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 3]], -1.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25], -1.3979400086720373`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20], -1.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50, 3]], -1.2218487496163561`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 7]], -1.1549019599857429`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[25, 2]], -1.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 9]], -1.0457574905606748`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-1, 
       FormBox["\"\\!\\(10\\^\\(-1\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[5]/Log[10], -0.6989700043360186], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 3]]/Log[10], -0.5228787452803375], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 2]]/Log[10], -0.39794000867203755`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[2]/Log[10], -0.30102999566398114`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 3]]/Log[10], -0.22184874961635634`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 7]]/Log[10], -0.15490195998574316`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 4]]/Log[10], -0.0969100130080564], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 9]]/Log[10], -0.04575749056067514], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{-6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500000], -5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 3]], -5.5228787452803365`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250000], -5.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200000], -5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500000, 3]], -5.221848749616355], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 7]], -5.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125000], -5.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 9]], -5.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50000], -4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 3]], -4.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25000], -4.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20000], -4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50000, 3]], -4.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 7]], -4.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[12500], -4.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 9]], -4.045757490560674], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[5000], -3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 3]], -3.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2500], -3.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2000], -3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[5000, 3]], -3.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 7]], -3.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[1250], -3.096910013008056], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 9]], -3.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500], -2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 3]], -2.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250], -2.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200], -2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500, 3]], -2.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 7]], -2.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125], -2.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 9]], -2.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50], -1.6989700043360185`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 3]], -1.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25], -1.3979400086720373`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20], -1.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50, 3]], -1.2218487496163561`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 7]], -1.1549019599857429`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[25, 2]], -1.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 9]], -1.0457574905606748`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-1, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[5]/Log[10], -0.6989700043360186], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 3]]/Log[10], -0.5228787452803375], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 2]]/Log[10], -0.39794000867203755`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[2]/Log[10], -0.30102999566398114`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 3]]/Log[10], -0.22184874961635634`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 7]]/Log[10], -0.15490195998574316`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 4]]/Log[10], -0.0969100130080564], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 9]]/Log[10], -0.04575749056067514], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847367368778*^9, 3.636847446536831*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{
    "Draw", " ", "the", " ", "plot", " ", "for", " ", "the", " ", "solar", 
     " ", "mass", " ", "squared", " ", "difference", " ", 
     RowBox[{"(", 
      RowBox[{"on", " ", "a", " ", "linear", " ", "scale"}], ")"}]}], " ", 
    "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Plot", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalDelta]m2sol", "[", "t", "]"}], "*", 
       RowBox[{"10", "^", "5"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
      RowBox[{"FrameLabel", "->", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
         "\"\<\!\(\*SubsuperscriptBox[\(\[CapitalDelta]m\), \(sol\), \(2\)]\) \
[\!\(\*SuperscriptBox[\(10\), \
\(-5\)]\)\[ThinSpace]\!\(\*SuperscriptBox[\(eV\), \(2\)]\)]\>\""}], "}"}]}], 
      ",", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"RGELogTicksLabeled", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"i", ",", "i"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", 
             RowBox[{"-", "100"}], ",", "100"}], "}"}]}], "]"}], ",", 
         RowBox[{"RGELogTicks", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"i", ",", "\"\<\>\""}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", 
             RowBox[{"-", "100"}], ",", "100"}], "}"}]}], "]"}]}], "}"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"All", ",", "All"}], "}"}]}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", "ForestGreen", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"Prolog", "\[Rule]", 
       RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
     "\[IndentingNewLine]", "]"}], ",", 
    RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.436085085621465*^9, 3.436085087304071*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwVk3k41AsXxydhxpqtRpIxg8H4/eY3CVGqb0xI2mWve9OGchPeSBSaVGQU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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 7.5},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubsuperscriptBox[\\(\[CapitalDelta]m\\), \\(sol\\), \
\\(2\\)]\\) [\\!\\(\\*SuperscriptBox[\\(10\\), \\(-5\\)]\\)\[ThinSpace]\\!\\(\
\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)]\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{-100, 
       FormBox[
        RowBox[{"-", "100"}], TraditionalForm]}, {-99, 
       FormBox[
        RowBox[{"-", "99"}], TraditionalForm]}, {-98, 
       FormBox[
        RowBox[{"-", "98"}], TraditionalForm]}, {-97, 
       FormBox[
        RowBox[{"-", "97"}], TraditionalForm]}, {-96, 
       FormBox[
        RowBox[{"-", "96"}], TraditionalForm]}, {-95, 
       FormBox[
        RowBox[{"-", "95"}], TraditionalForm]}, {-94, 
       FormBox[
        RowBox[{"-", "94"}], TraditionalForm]}, {-93, 
       FormBox[
        RowBox[{"-", "93"}], TraditionalForm]}, {-92, 
       FormBox[
        RowBox[{"-", "92"}], TraditionalForm]}, {-91, 
       FormBox[
        RowBox[{"-", "91"}], TraditionalForm]}, {-90, 
       FormBox[
        RowBox[{"-", "90"}], TraditionalForm]}, {-89, 
       FormBox[
        RowBox[{"-", "89"}], TraditionalForm]}, {-88, 
       FormBox[
        RowBox[{"-", "88"}], TraditionalForm]}, {-87, 
       FormBox[
        RowBox[{"-", "87"}], TraditionalForm]}, {-86, 
       FormBox[
        RowBox[{"-", "86"}], TraditionalForm]}, {-85, 
       FormBox[
        RowBox[{"-", "85"}], TraditionalForm]}, {-84, 
       FormBox[
        RowBox[{"-", "84"}], TraditionalForm]}, {-83, 
       FormBox[
        RowBox[{"-", "83"}], TraditionalForm]}, {-82, 
       FormBox[
        RowBox[{"-", "82"}], TraditionalForm]}, {-81, 
       FormBox[
        RowBox[{"-", "81"}], TraditionalForm]}, {-80, 
       FormBox[
        RowBox[{"-", "80"}], TraditionalForm]}, {-79, 
       FormBox[
        RowBox[{"-", "79"}], TraditionalForm]}, {-78, 
       FormBox[
        RowBox[{"-", "78"}], TraditionalForm]}, {-77, 
       FormBox[
        RowBox[{"-", "77"}], TraditionalForm]}, {-76, 
       FormBox[
        RowBox[{"-", "76"}], TraditionalForm]}, {-75, 
       FormBox[
        RowBox[{"-", "75"}], TraditionalForm]}, {-74, 
       FormBox[
        RowBox[{"-", "74"}], TraditionalForm]}, {-73, 
       FormBox[
        RowBox[{"-", "73"}], TraditionalForm]}, {-72, 
       FormBox[
        RowBox[{"-", "72"}], TraditionalForm]}, {-71, 
       FormBox[
        RowBox[{"-", "71"}], TraditionalForm]}, {-70, 
       FormBox[
        RowBox[{"-", "70"}], TraditionalForm]}, {-69, 
       FormBox[
        RowBox[{"-", "69"}], TraditionalForm]}, {-68, 
       FormBox[
        RowBox[{"-", "68"}], TraditionalForm]}, {-67, 
       FormBox[
        RowBox[{"-", "67"}], TraditionalForm]}, {-66, 
       FormBox[
        RowBox[{"-", "66"}], TraditionalForm]}, {-65, 
       FormBox[
        RowBox[{"-", "65"}], TraditionalForm]}, {-64, 
       FormBox[
        RowBox[{"-", "64"}], TraditionalForm]}, {-63, 
       FormBox[
        RowBox[{"-", "63"}], TraditionalForm]}, {-62, 
       FormBox[
        RowBox[{"-", "62"}], TraditionalForm]}, {-61, 
       FormBox[
        RowBox[{"-", "61"}], TraditionalForm]}, {-60, 
       FormBox[
        RowBox[{"-", "60"}], TraditionalForm]}, {-59, 
       FormBox[
        RowBox[{"-", "59"}], TraditionalForm]}, {-58, 
       FormBox[
        RowBox[{"-", "58"}], TraditionalForm]}, {-57, 
       FormBox[
        RowBox[{"-", "57"}], TraditionalForm]}, {-56, 
       FormBox[
        RowBox[{"-", "56"}], TraditionalForm]}, {-55, 
       FormBox[
        RowBox[{"-", "55"}], TraditionalForm]}, {-54, 
       FormBox[
        RowBox[{"-", "54"}], TraditionalForm]}, {-53, 
       FormBox[
        RowBox[{"-", "53"}], TraditionalForm]}, {-52, 
       FormBox[
        RowBox[{"-", "52"}], TraditionalForm]}, {-51, 
       FormBox[
        RowBox[{"-", "51"}], TraditionalForm]}, {-50, 
       FormBox[
        RowBox[{"-", "50"}], TraditionalForm]}, {-49, 
       FormBox[
        RowBox[{"-", "49"}], TraditionalForm]}, {-48, 
       FormBox[
        RowBox[{"-", "48"}], TraditionalForm]}, {-47, 
       FormBox[
        RowBox[{"-", "47"}], TraditionalForm]}, {-46, 
       FormBox[
        RowBox[{"-", "46"}], TraditionalForm]}, {-45, 
       FormBox[
        RowBox[{"-", "45"}], TraditionalForm]}, {-44, 
       FormBox[
        RowBox[{"-", "44"}], TraditionalForm]}, {-43, 
       FormBox[
        RowBox[{"-", "43"}], TraditionalForm]}, {-42, 
       FormBox[
        RowBox[{"-", "42"}], TraditionalForm]}, {-41, 
       FormBox[
        RowBox[{"-", "41"}], TraditionalForm]}, {-40, 
       FormBox[
        RowBox[{"-", "40"}], TraditionalForm]}, {-39, 
       FormBox[
        RowBox[{"-", "39"}], TraditionalForm]}, {-38, 
       FormBox[
        RowBox[{"-", "38"}], TraditionalForm]}, {-37, 
       FormBox[
        RowBox[{"-", "37"}], TraditionalForm]}, {-36, 
       FormBox[
        RowBox[{"-", "36"}], TraditionalForm]}, {-35, 
       FormBox[
        RowBox[{"-", "35"}], TraditionalForm]}, {-34, 
       FormBox[
        RowBox[{"-", "34"}], TraditionalForm]}, {-33, 
       FormBox[
        RowBox[{"-", "33"}], TraditionalForm]}, {-32, 
       FormBox[
        RowBox[{"-", "32"}], TraditionalForm]}, {-31, 
       FormBox[
        RowBox[{"-", "31"}], TraditionalForm]}, {-30, 
       FormBox[
        RowBox[{"-", "30"}], TraditionalForm]}, {-29, 
       FormBox[
        RowBox[{"-", "29"}], TraditionalForm]}, {-28, 
       FormBox[
        RowBox[{"-", "28"}], TraditionalForm]}, {-27, 
       FormBox[
        RowBox[{"-", "27"}], TraditionalForm]}, {-26, 
       FormBox[
        RowBox[{"-", "26"}], TraditionalForm]}, {-25, 
       FormBox[
        RowBox[{"-", "25"}], TraditionalForm]}, {-24, 
       FormBox[
        RowBox[{"-", "24"}], TraditionalForm]}, {-23, 
       FormBox[
        RowBox[{"-", "23"}], TraditionalForm]}, {-22, 
       FormBox[
        RowBox[{"-", "22"}], TraditionalForm]}, {-21, 
       FormBox[
        RowBox[{"-", "21"}], TraditionalForm]}, {-20, 
       FormBox[
        RowBox[{"-", "20"}], TraditionalForm]}, {-19, 
       FormBox[
        RowBox[{"-", "19"}], TraditionalForm]}, {-18, 
       FormBox[
        RowBox[{"-", "18"}], TraditionalForm]}, {-17, 
       FormBox[
        RowBox[{"-", "17"}], TraditionalForm]}, {-16, 
       FormBox[
        RowBox[{"-", "16"}], TraditionalForm]}, {-15, 
       FormBox[
        RowBox[{"-", "15"}], TraditionalForm]}, {-14, 
       FormBox[
        RowBox[{"-", "14"}], TraditionalForm]}, {-13, 
       FormBox[
        RowBox[{"-", "13"}], TraditionalForm]}, {-12, 
       FormBox[
        RowBox[{"-", "12"}], TraditionalForm]}, {-11, 
       FormBox[
        RowBox[{"-", "11"}], TraditionalForm]}, {-10, 
       FormBox[
        RowBox[{"-", "10"}], TraditionalForm]}, {-9, 
       FormBox[
        RowBox[{"-", "9"}], TraditionalForm]}, {-8, 
       FormBox[
        RowBox[{"-", "8"}], TraditionalForm]}, {-7, 
       FormBox[
        RowBox[{"-", "7"}], TraditionalForm]}, {-6, 
       FormBox[
        RowBox[{"-", "6"}], TraditionalForm]}, {-5, 
       FormBox[
        RowBox[{"-", "5"}], TraditionalForm]}, {-4, 
       FormBox[
        RowBox[{"-", "4"}], TraditionalForm]}, {-3, 
       FormBox[
        RowBox[{"-", "3"}], TraditionalForm]}, {-2, 
       FormBox[
        RowBox[{"-", "2"}], TraditionalForm]}, {-1, 
       FormBox[
        RowBox[{"-", "1"}], TraditionalForm]}, {0, 
       FormBox["0", TraditionalForm]}, {1, 
       FormBox["1", TraditionalForm]}, {2, 
       FormBox["2", TraditionalForm]}, {3, 
       FormBox["3", TraditionalForm]}, {4, 
       FormBox["4", TraditionalForm]}, {5, 
       FormBox["5", TraditionalForm]}, {6, 
       FormBox["6", TraditionalForm]}, {7, 
       FormBox["7", TraditionalForm]}, {8, 
       FormBox["8", TraditionalForm]}, {9, 
       FormBox["9", TraditionalForm]}, {10, 
       FormBox["10", TraditionalForm]}, {11, 
       FormBox["11", TraditionalForm]}, {12, 
       FormBox["12", TraditionalForm]}, {13, 
       FormBox["13", TraditionalForm]}, {14, 
       FormBox["14", TraditionalForm]}, {15, 
       FormBox["15", TraditionalForm]}, {16, 
       FormBox["16", TraditionalForm]}, {17, 
       FormBox["17", TraditionalForm]}, {18, 
       FormBox["18", TraditionalForm]}, {19, 
       FormBox["19", TraditionalForm]}, {20, 
       FormBox["20", TraditionalForm]}, {21, 
       FormBox["21", TraditionalForm]}, {22, 
       FormBox["22", TraditionalForm]}, {23, 
       FormBox["23", TraditionalForm]}, {24, 
       FormBox["24", TraditionalForm]}, {25, 
       FormBox["25", TraditionalForm]}, {26, 
       FormBox["26", TraditionalForm]}, {27, 
       FormBox["27", TraditionalForm]}, {28, 
       FormBox["28", TraditionalForm]}, {29, 
       FormBox["29", TraditionalForm]}, {30, 
       FormBox["30", TraditionalForm]}, {31, 
       FormBox["31", TraditionalForm]}, {32, 
       FormBox["32", TraditionalForm]}, {33, 
       FormBox["33", TraditionalForm]}, {34, 
       FormBox["34", TraditionalForm]}, {35, 
       FormBox["35", TraditionalForm]}, {36, 
       FormBox["36", TraditionalForm]}, {37, 
       FormBox["37", TraditionalForm]}, {38, 
       FormBox["38", TraditionalForm]}, {39, 
       FormBox["39", TraditionalForm]}, {40, 
       FormBox["40", TraditionalForm]}, {41, 
       FormBox["41", TraditionalForm]}, {42, 
       FormBox["42", TraditionalForm]}, {43, 
       FormBox["43", TraditionalForm]}, {44, 
       FormBox["44", TraditionalForm]}, {45, 
       FormBox["45", TraditionalForm]}, {46, 
       FormBox["46", TraditionalForm]}, {47, 
       FormBox["47", TraditionalForm]}, {48, 
       FormBox["48", TraditionalForm]}, {49, 
       FormBox["49", TraditionalForm]}, {50, 
       FormBox["50", TraditionalForm]}, {51, 
       FormBox["51", TraditionalForm]}, {52, 
       FormBox["52", TraditionalForm]}, {53, 
       FormBox["53", TraditionalForm]}, {54, 
       FormBox["54", TraditionalForm]}, {55, 
       FormBox["55", TraditionalForm]}, {56, 
       FormBox["56", TraditionalForm]}, {57, 
       FormBox["57", TraditionalForm]}, {58, 
       FormBox["58", TraditionalForm]}, {59, 
       FormBox["59", TraditionalForm]}, {60, 
       FormBox["60", TraditionalForm]}, {61, 
       FormBox["61", TraditionalForm]}, {62, 
       FormBox["62", TraditionalForm]}, {63, 
       FormBox["63", TraditionalForm]}, {64, 
       FormBox["64", TraditionalForm]}, {65, 
       FormBox["65", TraditionalForm]}, {66, 
       FormBox["66", TraditionalForm]}, {67, 
       FormBox["67", TraditionalForm]}, {68, 
       FormBox["68", TraditionalForm]}, {69, 
       FormBox["69", TraditionalForm]}, {70, 
       FormBox["70", TraditionalForm]}, {71, 
       FormBox["71", TraditionalForm]}, {72, 
       FormBox["72", TraditionalForm]}, {73, 
       FormBox["73", TraditionalForm]}, {74, 
       FormBox["74", TraditionalForm]}, {75, 
       FormBox["75", TraditionalForm]}, {76, 
       FormBox["76", TraditionalForm]}, {77, 
       FormBox["77", TraditionalForm]}, {78, 
       FormBox["78", TraditionalForm]}, {79, 
       FormBox["79", TraditionalForm]}, {80, 
       FormBox["80", TraditionalForm]}, {81, 
       FormBox["81", TraditionalForm]}, {82, 
       FormBox["82", TraditionalForm]}, {83, 
       FormBox["83", TraditionalForm]}, {84, 
       FormBox["84", TraditionalForm]}, {85, 
       FormBox["85", TraditionalForm]}, {86, 
       FormBox["86", TraditionalForm]}, {87, 
       FormBox["87", TraditionalForm]}, {88, 
       FormBox["88", TraditionalForm]}, {89, 
       FormBox["89", TraditionalForm]}, {90, 
       FormBox["90", TraditionalForm]}, {91, 
       FormBox["91", TraditionalForm]}, {92, 
       FormBox["92", TraditionalForm]}, {93, 
       FormBox["93", TraditionalForm]}, {94, 
       FormBox["94", TraditionalForm]}, {95, 
       FormBox["95", TraditionalForm]}, {96, 
       FormBox["96", TraditionalForm]}, {97, 
       FormBox["97", TraditionalForm]}, {98, 
       FormBox["98", TraditionalForm]}, {99, 
       FormBox["99", TraditionalForm]}, {100, 
       FormBox["100", TraditionalForm]}}, {{-100, 
       FormBox["\"\"", TraditionalForm]}, {-99, 
       FormBox["\"\"", TraditionalForm]}, {-98, 
       FormBox["\"\"", TraditionalForm]}, {-97, 
       FormBox["\"\"", TraditionalForm]}, {-96, 
       FormBox["\"\"", TraditionalForm]}, {-95, 
       FormBox["\"\"", TraditionalForm]}, {-94, 
       FormBox["\"\"", TraditionalForm]}, {-93, 
       FormBox["\"\"", TraditionalForm]}, {-92, 
       FormBox["\"\"", TraditionalForm]}, {-91, 
       FormBox["\"\"", TraditionalForm]}, {-90, 
       FormBox["\"\"", TraditionalForm]}, {-89, 
       FormBox["\"\"", TraditionalForm]}, {-88, 
       FormBox["\"\"", TraditionalForm]}, {-87, 
       FormBox["\"\"", TraditionalForm]}, {-86, 
       FormBox["\"\"", TraditionalForm]}, {-85, 
       FormBox["\"\"", TraditionalForm]}, {-84, 
       FormBox["\"\"", TraditionalForm]}, {-83, 
       FormBox["\"\"", TraditionalForm]}, {-82, 
       FormBox["\"\"", TraditionalForm]}, {-81, 
       FormBox["\"\"", TraditionalForm]}, {-80, 
       FormBox["\"\"", TraditionalForm]}, {-79, 
       FormBox["\"\"", TraditionalForm]}, {-78, 
       FormBox["\"\"", TraditionalForm]}, {-77, 
       FormBox["\"\"", TraditionalForm]}, {-76, 
       FormBox["\"\"", TraditionalForm]}, {-75, 
       FormBox["\"\"", TraditionalForm]}, {-74, 
       FormBox["\"\"", TraditionalForm]}, {-73, 
       FormBox["\"\"", TraditionalForm]}, {-72, 
       FormBox["\"\"", TraditionalForm]}, {-71, 
       FormBox["\"\"", TraditionalForm]}, {-70, 
       FormBox["\"\"", TraditionalForm]}, {-69, 
       FormBox["\"\"", TraditionalForm]}, {-68, 
       FormBox["\"\"", TraditionalForm]}, {-67, 
       FormBox["\"\"", TraditionalForm]}, {-66, 
       FormBox["\"\"", TraditionalForm]}, {-65, 
       FormBox["\"\"", TraditionalForm]}, {-64, 
       FormBox["\"\"", TraditionalForm]}, {-63, 
       FormBox["\"\"", TraditionalForm]}, {-62, 
       FormBox["\"\"", TraditionalForm]}, {-61, 
       FormBox["\"\"", TraditionalForm]}, {-60, 
       FormBox["\"\"", TraditionalForm]}, {-59, 
       FormBox["\"\"", TraditionalForm]}, {-58, 
       FormBox["\"\"", TraditionalForm]}, {-57, 
       FormBox["\"\"", TraditionalForm]}, {-56, 
       FormBox["\"\"", TraditionalForm]}, {-55, 
       FormBox["\"\"", TraditionalForm]}, {-54, 
       FormBox["\"\"", TraditionalForm]}, {-53, 
       FormBox["\"\"", TraditionalForm]}, {-52, 
       FormBox["\"\"", TraditionalForm]}, {-51, 
       FormBox["\"\"", TraditionalForm]}, {-50, 
       FormBox["\"\"", TraditionalForm]}, {-49, 
       FormBox["\"\"", TraditionalForm]}, {-48, 
       FormBox["\"\"", TraditionalForm]}, {-47, 
       FormBox["\"\"", TraditionalForm]}, {-46, 
       FormBox["\"\"", TraditionalForm]}, {-45, 
       FormBox["\"\"", TraditionalForm]}, {-44, 
       FormBox["\"\"", TraditionalForm]}, {-43, 
       FormBox["\"\"", TraditionalForm]}, {-42, 
       FormBox["\"\"", TraditionalForm]}, {-41, 
       FormBox["\"\"", TraditionalForm]}, {-40, 
       FormBox["\"\"", TraditionalForm]}, {-39, 
       FormBox["\"\"", TraditionalForm]}, {-38, 
       FormBox["\"\"", TraditionalForm]}, {-37, 
       FormBox["\"\"", TraditionalForm]}, {-36, 
       FormBox["\"\"", TraditionalForm]}, {-35, 
       FormBox["\"\"", TraditionalForm]}, {-34, 
       FormBox["\"\"", TraditionalForm]}, {-33, 
       FormBox["\"\"", TraditionalForm]}, {-32, 
       FormBox["\"\"", TraditionalForm]}, {-31, 
       FormBox["\"\"", TraditionalForm]}, {-30, 
       FormBox["\"\"", TraditionalForm]}, {-29, 
       FormBox["\"\"", TraditionalForm]}, {-28, 
       FormBox["\"\"", TraditionalForm]}, {-27, 
       FormBox["\"\"", TraditionalForm]}, {-26, 
       FormBox["\"\"", TraditionalForm]}, {-25, 
       FormBox["\"\"", TraditionalForm]}, {-24, 
       FormBox["\"\"", TraditionalForm]}, {-23, 
       FormBox["\"\"", TraditionalForm]}, {-22, 
       FormBox["\"\"", TraditionalForm]}, {-21, 
       FormBox["\"\"", TraditionalForm]}, {-20, 
       FormBox["\"\"", TraditionalForm]}, {-19, 
       FormBox["\"\"", TraditionalForm]}, {-18, 
       FormBox["\"\"", TraditionalForm]}, {-17, 
       FormBox["\"\"", TraditionalForm]}, {-16, 
       FormBox["\"\"", TraditionalForm]}, {-15, 
       FormBox["\"\"", TraditionalForm]}, {-14, 
       FormBox["\"\"", TraditionalForm]}, {-13, 
       FormBox["\"\"", TraditionalForm]}, {-12, 
       FormBox["\"\"", TraditionalForm]}, {-11, 
       FormBox["\"\"", TraditionalForm]}, {-10, 
       FormBox["\"\"", TraditionalForm]}, {-9, 
       FormBox["\"\"", TraditionalForm]}, {-8, 
       FormBox["\"\"", TraditionalForm]}, {-7, 
       FormBox["\"\"", TraditionalForm]}, {-6, 
       FormBox["\"\"", TraditionalForm]}, {-5, 
       FormBox["\"\"", TraditionalForm]}, {-4, 
       FormBox["\"\"", TraditionalForm]}, {-3, 
       FormBox["\"\"", TraditionalForm]}, {-2, 
       FormBox["\"\"", TraditionalForm]}, {-1, 
       FormBox["\"\"", TraditionalForm]}, {0, 
       FormBox["\"\"", TraditionalForm]}, {1, 
       FormBox["\"\"", TraditionalForm]}, {2, 
       FormBox["\"\"", TraditionalForm]}, {3, 
       FormBox["\"\"", TraditionalForm]}, {4, 
       FormBox["\"\"", TraditionalForm]}, {5, 
       FormBox["\"\"", TraditionalForm]}, {6, 
       FormBox["\"\"", TraditionalForm]}, {7, 
       FormBox["\"\"", TraditionalForm]}, {8, 
       FormBox["\"\"", TraditionalForm]}, {9, 
       FormBox["\"\"", TraditionalForm]}, {10, 
       FormBox["\"\"", TraditionalForm]}, {11, 
       FormBox["\"\"", TraditionalForm]}, {12, 
       FormBox["\"\"", TraditionalForm]}, {13, 
       FormBox["\"\"", TraditionalForm]}, {14, 
       FormBox["\"\"", TraditionalForm]}, {15, 
       FormBox["\"\"", TraditionalForm]}, {16, 
       FormBox["\"\"", TraditionalForm]}, {17, 
       FormBox["\"\"", TraditionalForm]}, {18, 
       FormBox["\"\"", TraditionalForm]}, {19, 
       FormBox["\"\"", TraditionalForm]}, {20, 
       FormBox["\"\"", TraditionalForm]}, {21, 
       FormBox["\"\"", TraditionalForm]}, {22, 
       FormBox["\"\"", TraditionalForm]}, {23, 
       FormBox["\"\"", TraditionalForm]}, {24, 
       FormBox["\"\"", TraditionalForm]}, {25, 
       FormBox["\"\"", TraditionalForm]}, {26, 
       FormBox["\"\"", TraditionalForm]}, {27, 
       FormBox["\"\"", TraditionalForm]}, {28, 
       FormBox["\"\"", TraditionalForm]}, {29, 
       FormBox["\"\"", TraditionalForm]}, {30, 
       FormBox["\"\"", TraditionalForm]}, {31, 
       FormBox["\"\"", TraditionalForm]}, {32, 
       FormBox["\"\"", TraditionalForm]}, {33, 
       FormBox["\"\"", TraditionalForm]}, {34, 
       FormBox["\"\"", TraditionalForm]}, {35, 
       FormBox["\"\"", TraditionalForm]}, {36, 
       FormBox["\"\"", TraditionalForm]}, {37, 
       FormBox["\"\"", TraditionalForm]}, {38, 
       FormBox["\"\"", TraditionalForm]}, {39, 
       FormBox["\"\"", TraditionalForm]}, {40, 
       FormBox["\"\"", TraditionalForm]}, {41, 
       FormBox["\"\"", TraditionalForm]}, {42, 
       FormBox["\"\"", TraditionalForm]}, {43, 
       FormBox["\"\"", TraditionalForm]}, {44, 
       FormBox["\"\"", TraditionalForm]}, {45, 
       FormBox["\"\"", TraditionalForm]}, {46, 
       FormBox["\"\"", TraditionalForm]}, {47, 
       FormBox["\"\"", TraditionalForm]}, {48, 
       FormBox["\"\"", TraditionalForm]}, {49, 
       FormBox["\"\"", TraditionalForm]}, {50, 
       FormBox["\"\"", TraditionalForm]}, {51, 
       FormBox["\"\"", TraditionalForm]}, {52, 
       FormBox["\"\"", TraditionalForm]}, {53, 
       FormBox["\"\"", TraditionalForm]}, {54, 
       FormBox["\"\"", TraditionalForm]}, {55, 
       FormBox["\"\"", TraditionalForm]}, {56, 
       FormBox["\"\"", TraditionalForm]}, {57, 
       FormBox["\"\"", TraditionalForm]}, {58, 
       FormBox["\"\"", TraditionalForm]}, {59, 
       FormBox["\"\"", TraditionalForm]}, {60, 
       FormBox["\"\"", TraditionalForm]}, {61, 
       FormBox["\"\"", TraditionalForm]}, {62, 
       FormBox["\"\"", TraditionalForm]}, {63, 
       FormBox["\"\"", TraditionalForm]}, {64, 
       FormBox["\"\"", TraditionalForm]}, {65, 
       FormBox["\"\"", TraditionalForm]}, {66, 
       FormBox["\"\"", TraditionalForm]}, {67, 
       FormBox["\"\"", TraditionalForm]}, {68, 
       FormBox["\"\"", TraditionalForm]}, {69, 
       FormBox["\"\"", TraditionalForm]}, {70, 
       FormBox["\"\"", TraditionalForm]}, {71, 
       FormBox["\"\"", TraditionalForm]}, {72, 
       FormBox["\"\"", TraditionalForm]}, {73, 
       FormBox["\"\"", TraditionalForm]}, {74, 
       FormBox["\"\"", TraditionalForm]}, {75, 
       FormBox["\"\"", TraditionalForm]}, {76, 
       FormBox["\"\"", TraditionalForm]}, {77, 
       FormBox["\"\"", TraditionalForm]}, {78, 
       FormBox["\"\"", TraditionalForm]}, {79, 
       FormBox["\"\"", TraditionalForm]}, {80, 
       FormBox["\"\"", TraditionalForm]}, {81, 
       FormBox["\"\"", TraditionalForm]}, {82, 
       FormBox["\"\"", TraditionalForm]}, {83, 
       FormBox["\"\"", TraditionalForm]}, {84, 
       FormBox["\"\"", TraditionalForm]}, {85, 
       FormBox["\"\"", TraditionalForm]}, {86, 
       FormBox["\"\"", TraditionalForm]}, {87, 
       FormBox["\"\"", TraditionalForm]}, {88, 
       FormBox["\"\"", TraditionalForm]}, {89, 
       FormBox["\"\"", TraditionalForm]}, {90, 
       FormBox["\"\"", TraditionalForm]}, {91, 
       FormBox["\"\"", TraditionalForm]}, {92, 
       FormBox["\"\"", TraditionalForm]}, {93, 
       FormBox["\"\"", TraditionalForm]}, {94, 
       FormBox["\"\"", TraditionalForm]}, {95, 
       FormBox["\"\"", TraditionalForm]}, {96, 
       FormBox["\"\"", TraditionalForm]}, {97, 
       FormBox["\"\"", TraditionalForm]}, {98, 
       FormBox["\"\"", TraditionalForm]}, {99, 
       FormBox["\"\"", TraditionalForm]}, {100, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.63684736752415*^9, 3.636847447199079*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"(*", " ", 
    RowBox[{
    "Draw", " ", "the", " ", "plot", " ", "for", " ", "the", " ", 
     "atmospheric", " ", "mass", " ", "squared", " ", "difference", " ", 
     RowBox[{"(", 
      RowBox[{"on", " ", "a", " ", "linear", " ", "scale"}], ")"}]}], " ", 
    "*)"}],
   FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Plot", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalDelta]m2atm", "[", "t", "]"}], "*", 
       RowBox[{"10", "^", "3"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
      RowBox[{"FrameLabel", "->", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
         "\"\<\!\(\*SubsuperscriptBox[\(\[CapitalDelta]m\), \(atm\), \(2\)]\) \
[\!\(\*SuperscriptBox[\(10\), \
\(-3\)]\)\[ThinSpace]\!\(\*SuperscriptBox[\(eV\), \(2\)]\)]\>\""}], "}"}]}], 
      ",", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"RGELogTicksLabeled", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "*", "0.2"}], ",", 
             RowBox[{"PaddedForm", "[", 
              RowBox[{
               RowBox[{"i", "*", "0.2"}], ",", 
               RowBox[{"{", 
                RowBox[{"2", ",", "1"}], "}"}]}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", 
             RowBox[{"-", "50"}], ",", "50"}], "}"}]}], "]"}], ",", 
         RowBox[{"RGELogTicks", "[", 
          RowBox[{"2", ",", "19"}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "*", "0.2"}], ",", "\"\<\>\""}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "50"}], "}"}]}], "]"}]}], "}"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"All", ",", "All"}], "}"}]}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", "Blue", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"Prolog", "\[Rule]", 
       RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
     "\[IndentingNewLine]", "]"}], ",", 
    RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.436085094391677*^9, 3.436085097838912*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k4lekbB/DjvIfj4LzvSYtdyhbeo5N2Y/mSkEhTSdmaZH5Ro0WKlIQT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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 2.1},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubsuperscriptBox[\\(\[CapitalDelta]m\\), \\(atm\\), \
\\(2\\)]\\) [\\!\\(\\*SuperscriptBox[\\(10\\), \\(-3\\)]\\)\[ThinSpace]\\!\\(\
\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)]\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{-10., 
       FormBox[
        TagBox[
         InterpretationBox["\"-10.0\"", -10., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-9.8\"", -9.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.600000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-9.6\"", -9.600000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-9.4\"", -9.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-9.2\"", -9.200000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9., 
       FormBox[
        TagBox[
         InterpretationBox["\"-9.0\"", -9., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.8\"", -8.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.6\"", -8.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.4\"", -8.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-8.2\"", -8.200000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8., 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.0\"", -8., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-7.8\"", -7.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-7.6\"", -7.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-7.4\"", -7.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-7.2\"", -7.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7., 
       FormBox[
        TagBox[
         InterpretationBox["\"-7.0\"", -7., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-6.8\"", -6.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-6.6\"", -6.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-6.4\"", -6.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-6.2\"", -6.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6., 
       FormBox[
        TagBox[
         InterpretationBox["\"-6.0\"", -6., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-5.8\"", -5.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-5.6\"", -5.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-5.4\"", -5.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-5.2\"", -5.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5., 
       FormBox[
        TagBox[
         InterpretationBox["\"-5.0\"", -5., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-4.8\"", -4.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-4.6\"", -4.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-4.4\"", -4.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-4.2\"", -4.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4., 
       FormBox[
        TagBox[
         InterpretationBox["\"-4.0\"", -4., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-3.8\"", -3.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-3.6\"", -3.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-3.4\"", -3.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-3.2\"", -3.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3., 
       FormBox[
        TagBox[
         InterpretationBox["\"-3.0\"", -3., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-2.8\"", -2.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-2.6\"", -2.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-2.4\"", -2.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-2.2\"", -2.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2., 
       FormBox[
        TagBox[
         InterpretationBox["\"-2.0\"", -2., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-1.8\"", -1.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-1.6\"", -1.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.4000000000000001`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-1.4\"", -1.4000000000000001`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.2000000000000002`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-1.2\"", -1.2000000000000002`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1., 
       FormBox[
        TagBox[
         InterpretationBox["\"-1.0\"", -1., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-0.8\"", -0.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.6000000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-0.6\"", -0.6000000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-0.4\"", -0.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-0.2\"", -0.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0., 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.0\"", 0., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.2\"", 0.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.4\"", 0.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.6000000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.6\"", 0.6000000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.8\"", 0.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1., 
       FormBox[
        TagBox[
         InterpretationBox["\" 1.0\"", 1., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.2000000000000002`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 1.2\"", 1.2000000000000002`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.4000000000000001`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 1.4\"", 1.4000000000000001`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 1.6\"", 1.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 1.8\"", 1.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2., 
       FormBox[
        TagBox[
         InterpretationBox["\" 2.0\"", 2., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 2.2\"", 2.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 2.4\"", 2.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 2.6\"", 2.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 2.8\"", 2.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3., 
       FormBox[
        TagBox[
         InterpretationBox["\" 3.0\"", 3., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 3.2\"", 3.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 3.4\"", 3.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 3.6\"", 3.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 3.8\"", 3.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4., 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.0\"", 4., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.2\"", 4.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.4\"", 4.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 4.6\"", 4.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.8\"", 4.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5., 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.0\"", 5., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.2\"", 5.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.4\"", 5.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 5.6\"", 5.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.8\"", 5.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6., 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.0\"", 6., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.2\"", 6.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.4\"", 6.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 6.6\"", 6.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.8\"", 6.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7., 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.0\"", 7., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.2\"", 7.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.4\"", 7.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 7.6\"", 7.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.8\"", 7.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8., 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.0\"", 8., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.2\"", 8.200000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.4\"", 8.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.6\"", 8.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.8\"", 8.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9., 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.0\"", 9., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.2\"", 9.200000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.4\"", 9.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.600000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.6\"", 9.600000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.8\"", 9.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {10., 
       FormBox[
        TagBox[
         InterpretationBox["\" 10.0\"", 10., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}}, {{0., 
       FormBox["\"\"", TraditionalForm]}, {0.2, 
       FormBox["\"\"", TraditionalForm]}, {0.4, 
       FormBox["\"\"", TraditionalForm]}, {0.6000000000000001, 
       FormBox["\"\"", TraditionalForm]}, {0.8, 
       FormBox["\"\"", TraditionalForm]}, {1., 
       FormBox["\"\"", TraditionalForm]}, {1.2000000000000002`, 
       FormBox["\"\"", TraditionalForm]}, {1.4000000000000001`, 
       FormBox["\"\"", TraditionalForm]}, {1.6, 
       FormBox["\"\"", TraditionalForm]}, {1.8, 
       FormBox["\"\"", TraditionalForm]}, {2., 
       FormBox["\"\"", TraditionalForm]}, {2.2, 
       FormBox["\"\"", TraditionalForm]}, {2.4000000000000004`, 
       FormBox["\"\"", TraditionalForm]}, {2.6, 
       FormBox["\"\"", TraditionalForm]}, {2.8000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {3., 
       FormBox["\"\"", TraditionalForm]}, {3.2, 
       FormBox["\"\"", TraditionalForm]}, {3.4000000000000004`, 
       FormBox["\"\"", TraditionalForm]}, {3.6, 
       FormBox["\"\"", TraditionalForm]}, {3.8000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {4., 
       FormBox["\"\"", TraditionalForm]}, {4.2, 
       FormBox["\"\"", TraditionalForm]}, {4.4, 
       FormBox["\"\"", TraditionalForm]}, {4.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {4.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {5., 
       FormBox["\"\"", TraditionalForm]}, {5.2, 
       FormBox["\"\"", TraditionalForm]}, {5.4, 
       FormBox["\"\"", TraditionalForm]}, {5.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {5.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {6., 
       FormBox["\"\"", TraditionalForm]}, {6.2, 
       FormBox["\"\"", TraditionalForm]}, {6.4, 
       FormBox["\"\"", TraditionalForm]}, {6.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {6.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {7., 
       FormBox["\"\"", TraditionalForm]}, {7.2, 
       FormBox["\"\"", TraditionalForm]}, {7.4, 
       FormBox["\"\"", TraditionalForm]}, {7.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {7.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {8., 
       FormBox["\"\"", TraditionalForm]}, {8.200000000000001, 
       FormBox["\"\"", TraditionalForm]}, {8.4, 
       FormBox["\"\"", TraditionalForm]}, {8.6, 
       FormBox["\"\"", TraditionalForm]}, {8.8, 
       FormBox["\"\"", TraditionalForm]}, {9., 
       FormBox["\"\"", TraditionalForm]}, {9.200000000000001, 
       FormBox["\"\"", TraditionalForm]}, {9.4, 
       FormBox["\"\"", TraditionalForm]}, {9.600000000000001, 
       FormBox["\"\"", TraditionalForm]}, {9.8, 
       FormBox["\"\"", TraditionalForm]}, {10., 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847367652995*^9, 3.636847447793745*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{956, 1041},
WindowMargins->{{Automatic, 0}, {Automatic, 18}},
PrintingPageRange->{Automatic, Automatic},
PageHeaderLines->{Inherited, False},
PrintingOptions->{"FirstPageFooter"->False,
"FirstPageHeader"->False,
"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{612, 792},
"PostScriptOutputFile":>"RGEPlots.nb.ps"},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
FrontEndVersion->"10.0 for Linux x86 (64-bit) (December 4, 2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 74, 0, 145, "Title"],
Cell[644, 24, 257, 6, 50, "Text"],
Cell[904, 32, 313, 9, 55, "Input"],
Cell[1220, 43, 696, 17, 121, "Input"],
Cell[1919, 62, 6250, 166, 785, "Input"],
Cell[CellGroupData[{
Cell[8194, 232, 4326, 110, 341, "Input"],
Cell[12523, 344, 51687, 874, 299, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64247, 1223, 4295, 110, 341, "Input"],
Cell[68545, 1335, 50336, 852, 295, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118918, 2192, 4792, 121, 385, "Input"],
Cell[123713, 2315, 61323, 1108, 292, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[185073, 3428, 4128, 104, 377, "Input"],
Cell[189204, 3534, 59826, 1039, 287, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[249067, 4578, 2502, 63, 243, "Input"],
Cell[251572, 4643, 64001, 1211, 297, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[315610, 5859, 2733, 69, 265, "Input"],
Cell[318346, 5930, 62815, 1193, 326, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
